@@ -200,14 +200,11 @@ impl<T: Storage> Color<T> {
     #[must_use]
     pub fn lighten(self, percent: f64) -> Self {
         assert_percent(percent);
-
-        // First convert to HSL
-        let (h, s, mut l, a) = self.to_hsla();
-
-        // Increase the lightness and ensure we don't go over 100.0
-        l = (l + percent * 100.0).min(100.0);
-
-        Color::from_hsla(h, s, l, a)
+        self.map_hsla(|h, s, mut l, a| {
+            // Increase the lightness and ensure we don't go over 1.0
+            l = (l + percent).min(1.0);
+            (h, s, l, a)
+        })
     }
 
     /// Darkens the color by translating to HSL color space then adjusting the lightness value.
@@ -218,14 +215,11 @@ impl<T: Storage> Color<T> {
     #[must_use]
     pub fn darken(self, percent: f64) -> Self {
         assert_percent(percent);
-
-        // First convert to HSL
-        let (h, s, mut l, a) = self.to_hsla();
-
-        // Decrease the lightness and ensure we don't go below 0.0
-        l = (l - percent * 100.0).max(0.0);
-
-        Color::from_hsla(h, s, l, a)
+        self.map_hsla(|h, s, mut l, a| {
+            // Decrease the lightness and ensure we don't go below 0.0
+            l = (l - percent).max(0.0);
+            (h, s, l, a)
+        })
     }
 
     /// Increases saturation of the color by translating to HSL color space then adjusting the
@@ -237,14 +231,11 @@ impl<T: Storage> Color<T> {
     #[must_use]
     pub fn saturate(self, percent: f64) -> Self {
         assert_percent(percent);
-
-        // First convert to HSL
-        let (h, mut s, l, a) = self.to_hsla();
-
-        // Increase the saturation and ensure we don't go over 100.0
-        s = (s + percent * 100.0).min(100.0);
-
-        Color::from_hsla(h, s, l, a)
+        self.map_hsla(|h, mut s, l, a| {
+            // Increase the saturation and ensure we don't go over 1.0
+            s = (s + percent).min(1.0);
+            (h, s, l, a)
+        })
     }
 
     /// Decreases saturation of the color by translating to HSL color space then adjusting the
@@ -256,33 +247,22 @@ impl<T: Storage> Color<T> {
     #[must_use]
     pub fn desaturate(self, percent: f64) -> Self {
         assert_percent(percent);
-
-        // First convert to HSL
-        let (h, mut s, l, a) = self.to_hsla();
-
-        // Decrease the saturation and ensure we don't go below 0.0
-        s = (s - percent * 100.0).max(0.0);
-
-        Color::from_hsla(h, s, l, a)
+        self.map_hsla(|h, mut s, l, a| {
+            // Decrease the saturation and ensure we don't go below 0.0
+            s = (s - percent).max(0.0);
+            (h, s, l, a)
+        })
     }
 
     /// Rotate the hue by translating to HSL color space then adjusting the hue value. Takes a
     /// value between `0.0` and `360.0`.
-    ///
-    /// # Panics
-    ///
-    /// Panics if `amount` is less than `0.0`
     #[must_use]
     pub fn rotate_hue(self, amount: f64) -> Self {
-        assert!(amount >= 0.0, "amount must be greater than 0.0");
-
-        // First convert to HSL
-        let (mut h, s, l, a) = self.to_hsla();
-
-        // Add the amount and ensure the value is a positive number between 0.0 and 360.0
-        h = ((h + amount) % 360.0 + 360.0) % 360.0;
-
-        Color::from_hsla(h, s, l, a)
+        self.map_hsla(|mut h, s, l, a| {
+            // Add the amount and ensure the value is a positive number between 0.0 and 360.0
+            h = ((h + amount) % 360.0 + 360.0) % 360.0;
+            (h, s, l, a)
+        })
     }
 
     /// Returns the underlying `u32`.
@@ -306,23 +286,49 @@ impl<T: Storage> Color<T> {
         self.brightness() > 0.5
     }
 
-    /// Modify the individual channels.
+    /// Maps `(r1, g1, b1, a1)` to `(r2, g2, b2, a2)` by applying a function to the channels.
     ///
     /// # Examples
     ///
     /// ```
-    /// let color = inku::Color::<inku::RGBA>::new(0x00000011);
+    /// # use inku::{Color, RGBA};
+    /// let color = Color::<RGBA>::new(0x00000011);
     /// assert_eq!(
     ///     color.map(|r, g, b, a| (r, g, b, a + 0x22)).to_u32(),
     ///     0x00000033
     /// );
     /// ```
-    pub fn map(&self, f: fn(u8, u8, u8, u8) -> (u8, u8, u8, u8)) -> Self {
+    ///
+    /// ```
+    /// # use inku::{Color,  ZRGB};
+    /// const F: fn(u8, u8, u8, u8) -> (u8, u8, u8, u8) = |r, g, b, a| {
+    ///     assert_eq!(r, 0x22);
+    ///     assert_eq!(g, 0x33);
+    ///     assert_eq!(b, 0x44);
+    ///     assert_eq!(a, 0x00);
+    ///     (1, 2, 3, 4)
+    /// };
+    /// let color = Color::<ZRGB>::new(0x11223344);
+    /// assert_eq!(color.map(F).to_u32(), 0x00010203);
+    /// ```
+    pub fn map<F>(&self, f: F) -> Self
+    where
+        F: Fn(u8, u8, u8, u8) -> (u8, u8, u8, u8),
+    {
         let (r, g, b, a) = self.to_rgba();
         let (r, g, b, a) = f(r, g, b, a);
         Self::from_rgba(r, g, b, a)
     }
 
+    fn map_hsla<F>(&self, f: F) -> Self
+    where
+        F: Fn(f64, f64, f64, f64) -> (f64, f64, f64, f64),
+    {
+        let (h, s, l, a) = self.to_hsla();
+        let (h, s, l, a) = f(h, s, l, a);
+        Color::from_hsla(h, s, l, a)
+    }
+
     fn to_rgba(self) -> (u8, u8, u8, u8) {
         let (r, g, b, a) = T::decode(self.0);
         (r, g, b, a)
@@ -336,19 +342,19 @@ impl<T: Storage> Color<T> {
     // colors in HSLA you're probably better off keeping your colors in HSLA so you don't lose
     // fidelity.
 
-    fn to_hsla(self) -> (f64, f64, f64, u8) {
+    fn to_hsla(self) -> (f64, f64, f64, f64) {
         let (r, g, b, a) = self.to_rgba();
-        let (h, s, l) = rgb_to_hsl(r, g, b);
+        let (h, s, l, a) = rgba_to_hsla(r, g, b, a);
         (h, s, l, a)
     }
 
-    fn from_hsla(h: f64, s: f64, l: f64, a: u8) -> Self {
-        let (r, g, b) = hsl_to_rgb(h, s, l);
-        Self::from_rgba(r as u8, g as u8, b as u8, a)
+    fn from_hsla(h: f64, s: f64, l: f64, a: f64) -> Self {
+        let (r, g, b, a) = hsla_to_rgba(h, s, l, a);
+        Self::from_rgba(r as u8, g as u8, b as u8, a as u8)
     }
 }
 
-#[cfg(not(test))]
+#[cfg(not(any(test, feature = "color_debug")))]
 impl<T: Storage> fmt::Debug for Color<T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let storage = format!("{}", std::any::type_name::<T>())
@@ -362,6 +368,31 @@ impl<T: Storage> fmt::Debug for Color<T> {
     }
 }
 
+#[cfg(any(test, feature = "color_debug"))]
+impl<T: Storage> fmt::Debug for Color<T> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        use crossterm::style;
+
+        let fg = style::SetForegroundColor(if self.is_light() {
+            style::Color::Black
+        } else {
+            style::Color::White
+        });
+        let (r, g, b, _a) = self.to_rgba();
+        let bg = style::SetBackgroundColor(style::Color::Rgb { r, g, b });
+
+        let storage = format!("{}", std::any::type_name::<T>())
+            .split("::")
+            .last()
+            .expect("no type name")
+            .to_owned();
+
+        write!(f, "Color<{}>({}{}", storage, fg, bg)?;
+        T::write_hex(f, self.0)?;
+        write!(f, "{})", style::ResetColor)
+    }
+}
+
 impl<T: Storage> From<u32> for Color<T> {
     fn from(color: u32) -> Self {
         Self::new(color)
@@ -375,6 +406,13 @@ impl<T: Storage> From<(u8, u8, u8)> for Color<T> {
     }
 }
 
+impl<T: Storage> From<(u8, u8, u8, u8)> for Color<T> {
+    fn from(rgba: (u8, u8, u8, u8)) -> Self {
+        let (r, g, b, a) = rgba;
+        Self::from_rgba(r, g, b, a)
+    }
+}
+
 impl<T: Storage> From<Color<T>> for u32 {
     fn from(color: Color<T>) -> u32 {
         color.to_u32()
@@ -388,6 +426,12 @@ impl<T: Storage> From<Color<T>> for (u8, u8, u8) {
     }
 }
 
+impl<T: Storage> From<Color<T>> for (u8, u8, u8, u8) {
+    fn from(color: Color<T>) -> (u8, u8, u8, u8) {
+        color.to_rgba()
+    }
+}
+
 fn assert_percent(percent: f64) {
     assert!(
         (0.0..=1.0).contains(&percent),
@@ -397,24 +441,13 @@ fn assert_percent(percent: f64) {
 }
 
 // https://css-tricks.com/converting-color-spaces-in-javascript/#hsl-to-rgb
-fn hsl_to_rgb(h: f64, mut s: f64, mut l: f64) -> (u8, u8, u8) {
+fn hsla_to_rgba(h: f64, s: f64, l: f64, mut a: f64) -> (u8, u8, u8, u8) {
     debug_assert!(
         (0.0..=360.0).contains(&h),
         "h must be between 0.0 and 360.0"
     );
-    debug_assert!(
-        (0.0..=100.0).contains(&s),
-        "s must be between 0.0 and 100.0"
-    );
-    debug_assert!(
-        (0.0..=100.0).contains(&l),
-        "l must be between 0.0 and 100.0"
-    );
-
-    // Since we'll use a range of 0.0-100.0 for the saturation and lightness, the first step is to
-    // divide them by 100.0 to values between 0.0 and 1.0.
-    s /= 100.0;
-    l /= 100.0;
+    debug_assert!((0.0..=1.0).contains(&s), "s must be between 0.0 and 1.0");
+    debug_assert!((0.0..=1.0).contains(&l), "l must be between 0.0 and 1.0");
 
     // Next, we find chroma (c), which is color intensity
     let c = (1.0 - (2.0 * l - 1.0).abs()) * s;
@@ -443,20 +476,23 @@ fn hsl_to_rgb(h: f64, mut s: f64, mut l: f64) -> (u8, u8, u8) {
         unreachable!();
     };
 
-    // To get the final RGB value, we add m to each channel, multiply it by 255
+    // To get the final RGBA value, we add m to each channel, multiply it by 255
     r = (r + m) * 255.0;
     g = (g + m) * 255.0;
     b = (b + m) * 255.0;
+    a = a * 255.0;
 
-    (r as u8, g as u8, b as u8)
+    (r as u8, g as u8, b as u8, a as u8)
 }
 
 // https://css-tricks.com/converting-color-spaces-in-javascript/#rgb-to-hsl
-fn rgb_to_hsl(r: u8, g: u8, b: u8) -> (f64, f64, f64) {
-    // First, we must divide the red, green, and blue by 255 to use values between 0.0 and 1.0.
+fn rgba_to_hsla(r: u8, g: u8, b: u8, a: u8) -> (f64, f64, f64, f64) {
+    // First, we must divide the red, green, and blue, and alpha by 255 to use values between 0.0
+    // and 1.0.
     let r = r as f64 / 255.0;
     let g = g as f64 / 255.0;
     let b = b as f64 / 255.0;
+    let a = a as f64 / 255.0;
 
     // Then we find the minimum and maximum of those values (c_min and c_max) as well as the
     // difference between them (delta).
@@ -488,54 +524,25 @@ fn rgb_to_hsl(r: u8, g: u8, b: u8) -> (f64, f64, f64) {
     }
 
     // Calculate lightness
-    let mut l = (c_max + c_min) / 2.0;
+    let l = (c_max + c_min) / 2.0;
 
     // Calculate saturation
-    let mut s = if delta == 0.0 {
+    let s = if delta == 0.0 {
         0.0
     } else {
         // For rounding issues we need to ensure we stay below 1.0
         (delta / (1.0 - (2.0 * l - 1.0).abs())).min(1.0)
     };
 
-    // Multiply l and s by 100
-    s *= 100.0;
-    l *= 100.0;
+    debug_assert!(s <= 1.0);
+    debug_assert!(l <= 1.0);
 
-    debug_assert!(s <= 100.0);
-    debug_assert!(l <= 100.0);
-
-    (h, s, l)
+    (h, s, l, a)
 }
 
 #[cfg(test)]
 mod tests {
-    use super::{Color, Storage, RGBA, ZRGB};
-    use std::fmt;
-
-    impl<T: Storage> fmt::Debug for Color<T> {
-        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-            use crossterm::style;
-
-            let fg = style::SetForegroundColor(if self.is_light() {
-                style::Color::Black
-            } else {
-                style::Color::White
-            });
-            let (r, g, b, _a) = self.to_rgba();
-            let bg = style::SetBackgroundColor(style::Color::Rgb { r, g, b });
-
-            let storage = format!("{}", std::any::type_name::<T>())
-                .split("::")
-                .last()
-                .expect("no type name")
-                .to_owned();
-
-            write!(f, "Color<{}>({}{}", storage, fg, bg)?;
-            T::write_hex(f, self.0)?;
-            write!(f, "{})", style::ResetColor)
-        }
-    }
+    use super::*;
 
     #[test]
     fn lighten() {
@@ -630,7 +637,7 @@ mod tests {
 
     #[test]
     fn to_hsla() {
-        let color = Color::<ZRGB>::new(0xff966432);
+        let color = Color::<RGBA>::new(0x96643233);
         let (h, s, l, a) = color.to_hsla();
 
         let assert_float = |a: f64, b: f64| {
@@ -639,11 +646,11 @@ mod tests {
         };
 
         assert_float(29.999999999999996, h);
-        assert_float(50.000000000000014, s);
-        assert_float(39.215686274509810, l);
-        assert_eq!(0, a);
+        assert_float(0.50000000000000014, s);
+        assert_float(0.39215686274509810, l);
+        assert_float(0.2, a);
 
-        // Ensure saturation is within 0.0 and 100.0
+        // Ensure saturation is within 0.0 and 1.0
         Color::<ZRGB>::new(0xff2009).to_hsla();
     }
 
@@ -679,6 +686,9 @@ mod tests {
                 .to_u32(),
             0x01020304
         );
+
+        const RED: fn(u8, u8, u8, u8) -> (u8, u8, u8, u8) = |_r, g, b, a| (255, g, b, a);
+        assert_eq!(color.map(RED).to_u32(), 0xff000000);
     }
 
     #[test]
@@ -695,4 +705,15 @@ mod tests {
         eprintln!("{:?}", Color::<ZRGB>::new(0xff_facade));
         eprintln!("{:?}", Color::<RGBA>::new(0xfacade_ff));
     }
+
+    #[test]
+    fn test_rgba_to_hsla() {
+        let (h, s, l, a) = rgba_to_hsla(1, 2, 3, 4);
+        let (r, g, b, a) = hsla_to_rgba(h, s, l, a);
+
+        assert_eq!(r, 1);
+        assert_eq!(g, 2);
+        assert_eq!(b, 3);
+        assert_eq!(a, 4);
+    }
 }
